
require 'wxruby'
require 'delegate'

module Ridge
	module GUI
		def message(str)
			Wx::MessageDialog.new(nil, str, 'Message', Wx::CENTRE | Wx::OK).show_modal()
		end
		module_function :message
		
		class Application
			def initialize()
				@native = Wx::App.new()
			end
			attr :native
		end

		class WxWidget < Widget
			private
			DefaultPosition = Wx::Point.new(-1, -1)
			DefaultSize = Wx::Size.new(-1, -1)
			
			def create_widget(parent=nil)
			end

			@@application = nil
			def init_widget(parent=nil, *args)
				parent = @parent if parent.nil? && @parent!=nil
				@@application = Application.new() unless @@application
				@native = create_widget(parent, *args)
			end

			def set_size(clientsize = false)
				size = nil
				if clientsize then
					size = @native.get_client_size
				else
					size = @native.get_size
				end
				size.set_width(Length.to_px(properties['width'])) if properties['width']
				size.set_height(Length.to_px(properties['height'])) if properties['height']
				if clientsize then
					@native.set_auto_layout(true)
					@native.set_client_size(size)
				else
					@native.set_size(size)
				end
				pos = @native.get_position
				x = pos.x
				y = pos.y
				x = Length.to_px(properties['left']) if properties['left']
				y = Length.to_px(properties['top']) if properties['top']
				@native.move(Wx::Point.new(x,y))
			end

			def set_layout(parent)
				parent.dock_widget(self, properties['dock']) if properties['dock']
			end

			def do_layout()
				return unless defined?(@docked_widgets)
				dock_edge = [nil, nil, nil, nil]
				current_sizer = nil
				method_add = nil
				@docked_widgets.each do |ctrl|
					sizer = nil
					next_is_add = nil
					flg_margin = 0
					flg_margin |= Wx::DIRECTION_LEFT   if dock_edge[0].nil?
					flg_margin |= Wx::DIRECTION_TOP    if dock_edge[1].nil?
					flg_margin |= Wx::DIRECTION_RIGHT  if dock_edge[2].nil?
					flg_margin |= Wx::DIRECTION_BOTTOM if dock_edge[3].nil?
					margin = Length.to_px(properties['margin'])
					padding = Length.to_px(properties['padding'])
					case ctrl[1]
					when 'top'
						sizer = Wx::BoxSizer.new(Wx::VERTICAL)
						flg_margin &= ~Wx::DIRECTION_BOTTOM
						dock_edge[1] = ctrl[0]
						sizer.add(ctrl[0].native, 0, Wx::GROW | flg_margin, margin)
						sizer.add(0, padding, 0)
						next_is_add = true
					when 'bottom'
						sizer = Wx::BoxSizer.new(Wx::VERTICAL)
						flg_margin &= ~Wx::DIRECTION_TOP
						dock_edge[3] = ctrl[0]
						sizer.add(ctrl[0].native, 0, Wx::GROW | flg_margin, margin)
						sizer.insert(0, 0, padding, 0)
						next_is_add = false
					when 'left'
						sizer = Wx::BoxSizer.new(Wx::HORIZONTAL)
						flg_margin &= ~Wx::DIRECTION_RIGHT
						dock_edge[0] = ctrl[0]
						sizer.add(ctrl[0].native, 0, Wx::GROW | flg_margin, margin)
						sizer.add(padding, 0, 0)
						next_is_add = true
					when 'right'
						sizer = Wx::BoxSizer.new(Wx::HORIZONTAL)
						flg_margin &= ~Wx::DIRECTION_LEFT
						dock_edge[2] = ctrl[0]
						sizer.add(ctrl[0].native, 0, Wx::GROW | flg_margin, margin)
						sizer.insert(0, padding, 0, 0)
						next_is_add = false
					when 'fill'
						sizer = Wx::BoxSizer.new(Wx::HORIZONTAL)
						sizer.add(ctrl[0].native, 1, Wx::GROW | flg_margin, margin)
						dock_edge[0] = dock_edge[1] = dock_edge[2] = dock_edge[3] = ctrl[0]
						next_is_add = true
					end
					if current_sizer then
						if method_add then
							current_sizer.add(sizer, 1, Wx::GROW)
						else
							current_sizer.insert(0, sizer, 1, Wx::GROW)
						end
						current_sizer = sizer
					else
						current_sizer = sizer
						native.set_sizer(sizer)
					end
					method_add = next_is_add
				end
				if current_sizer and not method_add then
					current_sizer.insert(0, 0, 0, 1, Wx::GROW)
				end
				native.set_auto_layout(true)
			end

			def set_events(*names)
				names.each do |name|
					if properties.include?(name) then
						events.add_handler(name) {|*args|
							resource.events.fire(properties[name], *args)
						}
					end
				end
			end

			def destroy()
				@native.destroy()
				@native = nil
			end

			def init_native(*args)
				init_widget(*args)
				events.fire('on_init', self, {})
			end
			
			def initialize(resource, properties, children)
				super
				@native = nil
				set_events('on_init')
			end
			
			public
			def dock_widget(widget, pos)
				@docked_widgets = [] unless defined?(@docked_widgets)
				@docked_widgets.push([widget, pos])
			end
			
			def native
				init_native if @native.nil?
				@native
			end

			def attach_to(parent)
				@parent = parent
				init_native(parent) if @native.nil?
			end

			def application()
				@@application
			end
		end
		
		module WxWindowedWidget
			def init_native(*args)
				super
				self.enabled = Boolean.parse(properties['enabled']) if properties['enabled']
				self.visible = Boolean.parse(properties['visible']) if properties['visible']
			end

			def enabled=(val)
				native.enable(val)
			end

			def enabled(val)
				native.is_enabled()
			end

			def visible=(val)
				native.show(val)
			end

			def visible()
				native.is_shown()
			end
		end
		
		class Window < WxWidget
			include WxWindowedWidget
			def create_widget(parent, style=Wx::DEFAULT_FRAME_STYLE)
				pnative = nil
				pnative = parent.native if parent
				window = Wx::Frame.new(pnative, -1, '', Wx::Point.new(-1, -1), Wx::Size.new(-1, -1), style)
				return window
			end

			public :children
			def init_widget(parent=nil, style=nil)
				unless style then
					style = Wx::DEFAULT_FRAME_STYLE
					style = Wx::SYSTEM_MENU | Wx::CAPTION | Wx::SIMPLE_BORDER if Boolean.parse(properties['fixed'])
				end
				super(parent, style)

				native.set_title(properties['text']) if properties['text']
				if properties['icon'] then
					icon = resource.image(properties['icon']).new
					native.set_icon(icon.native)
				end
				set_size(true)
				children.each do |child|
					child.attach_to(self)
				end
				do_layout()

				native.evt_close() {|event| 
					@closing = true
					args = { 'close'=>true }
					events.fire('on_close', self, args)
					if not args['close'] then
						@closing = false
						event.veto()
					else
						close()
					end
				}
				native.evt_show() {|event| events.fire('on_open', self, {}) if native.is_shown() }
				#native.evt_activate() {|event| events.fire('on_active', self, {})}
			end

			def initialize(resource, properties, children)
				super
				@wait_cursor = false
				@closing = false
				set_events('on_close', 'on_open', 'on_active')
			end
			
			def show()
				native.show()
			end

			def close()
				if @closing then
					native.destroy()
				else
					native.close()
				end
				@closing = false
			end
			public :children

			def run()
				self.show
				self.application.native.main_loop()
			end

			def wait_cursor()
				@wait_cursor
			end

			def wait_cursor=(val)
				if val then
					native.set_cursor(Wx::Cursor.new(Wx::CURSOR_WAIT)) if not @wait_cursor
				else
					native.set_cursor(Wx::Cursor.new()) if @wait_cursor
				end
				@wait_cursor = val
			end
		end
		Widgets['window'] = Window

		class Dialog < Window
			def initialize(resource, properties, children)
				super
				@attached = false
			end

			def create_widget(parent, style=Wx::DEFAULT_DIALOG_STYLE)
				pnative = nil
				pnative = parent.native if parent
				window = Wx::Dialog.new(pnative, -1, '', Wx::Point.new(-1, -1), Wx::Size.new(-1, -1), style)
				return window
			end

			public :children
			def init_widget(parent=nil, style=nil)
				unless style then
					style = Wx::DEFAULT_DIALOG_STYLE
					style |= Wx::RESIZE_BORDER if Boolean.parse(properties['resizable'])
				end
				super(parent, style)
			end
			
			def attach_to(parent)
				super
				@attached = true
			end

			def show()
				if @attached then
					ret = native.show_modal()
					return ret
				else
					native.show(true)
				end
			end

			def close(ret = 0)
				if @attached then
					native.end_modal(ret)
				else
					super
				end
				@closing = false
			end
		end
		Widgets['dialog'] = Dialog

		class Panel < WxWidget
			include WxWindowedWidget
			def create_widget(parent, style)
				Wx::Panel.new(parent.native, -1, DefaultPosition, DefaultSize, style)
			end

			public :children
			def init_widget(parent)
				style = Wx::TAB_TRAVERSAL
				style |= Wx::SUNKEN_BORDER if Boolean.parse(properties['edge'])
				super(parent, style)
				set_size(true)
				set_layout(parent)
				self.children.each do |child|
					child.attach_to(self)
				end
				do_layout()
			end
		end
		Widgets['panel'] = Panel

		class RowLayout < WxWidget
			include WxWindowedWidget
			def create_widget(parent, style)
				Wx::Panel.new(parent.native, -1, DefaultPosition, DefaultSize, style)
			end

			public :children
			def init_widget(parent)
				style = Wx::TAB_TRAVERSAL
				style |= Wx::SUNKEN_BORDER if Boolean.parse(properties['edge'])
				super(parent, style)
				set_size(true)
				set_layout(parent)
				self.children.each do |child|
					child.attach_to(self)
				end
				do_layout()
			end

			def do_layout()
				sizer = Wx::BoxSizer.new(Wx::VERTICAL)
				margin = Length.to_px(properties['margin'])
				padding = Length.to_px(properties['padding'])
				dock_edge = nil
				size = []
				size = properties['size'].split(' ').collect() {|sz| sz.to_i} if properties['size']
				children.each_with_index do |ctrl, i|
					flg_margin = Wx::DIRECTION_LEFT | Wx::DIRECTION_RIGHT
					if dock_edge.nil? then
						flg_margin |= Wx::DIRECTION_TOP
					else
						sizer.add(0, padding, 0)
					end
					dock_edge = ctrl
					propotion = size[i]
					propotion = 0 if propotion.nil?
					sizer.add(ctrl.native, propotion, Wx::GROW | flg_margin, margin)
				end
				sizer.add(0, margin, 0, Wx::GROW)
				native.set_sizer(sizer)
				native.set_auto_layout(true)
			end
			private :do_layout
		end
		Widgets['rowlayout'] = RowLayout

		class ColumnLayout < WxWidget
			include WxWindowedWidget
			def create_widget(parent, style)
				Wx::Panel.new(parent.native, -1, DefaultPosition, DefaultSize, style)
			end

			public :children
			def init_widget(parent)
				style = Wx::TAB_TRAVERSAL
				style |= Wx::SUNKEN_BORDER if Boolean.parse(properties['edge'])
				super(parent, style)
				set_size(true)
				set_layout(parent)
				self.children.each do |child|
					child.attach_to(self)
				end
				do_layout()
			end

			def do_layout()
				sizer = Wx::BoxSizer.new(Wx::HORIZONTAL)
				margin = Length.to_px(properties['margin'])
				padding = Length.to_px(properties['padding'])
				dock_edge = nil
				size = []
				size = properties['size'].split(' ').collect() {|sz| sz.to_i} if properties['size']
				children.each_with_index do |ctrl, i|
					flg_margin = Wx::DIRECTION_TOP | Wx::DIRECTION_BOTTOM
					if dock_edge.nil? then
						flg_margin |= Wx::DIRECTION_LEFT
					else
						sizer.add(padding, 0, 0)
					end
					dock_edge = ctrl
					propotion = size[i]
					propotion = 0 if propotion.nil?
					sizer.add(ctrl.native, propotion, Wx::GROW | flg_margin, margin)
				end
				sizer.add(margin, 0, 0, Wx::GROW)
				native.set_sizer(sizer)
				native.set_auto_layout(true)
			end
			private :do_layout
		end
		Widgets['columnlayout'] = ColumnLayout

		class ScrollPanel < WxWidget
			include WxWindowedWidget
			class WxScrolledWindow < Wx::ScrolledWindow
				def on_draw(dc)
				end
			end

			def create_widget(parent, style)
				WxScrolledWindow.new(parent.native, -1, DefaultPosition, DefaultSize, style)
			end

			public :children
			def init_widget(parent)
				style = Wx::TAB_TRAVERSAL | Wx::HSCROLL | Wx::VSCROLL
				style |= Wx::SUNKEN_BORDER if Boolean.parse(properties['edge'])
				super(parent, style)
				set_size(true)
				set_layout(parent)
        native.set_scroll_rate(20, 20)
				self.children.each do |child|
					child.attach_to(self)
				end
				do_layout()
			end
		end
		Widgets['scrollpanel'] = ScrollPanel

		class Button < WxWidget
			include WxWindowedWidget
			def initialize(resource, properties, children)
				super
				set_events('on_click')
			end

			def create_widget(parent)
				label = properties['text']
				label = '' unless label
				Wx::Button.new(parent.native, -1, label)
			end

			def init_widget(parent)
				super
				native.evt_button(native.get_id()) {|event| 
					events.fire('on_click', self, {})
				}
				set_size()
				set_layout(parent)
			end
		end
		Widgets['button'] = Button

		class Label < WxWidget
			include WxWindowedWidget
			def create_widget(parent)
				label = properties['text']
				label = '' unless label
				Wx::StaticText.new(parent.native, -1, label)
			end

			def init_widget(parent)
				super
				set_size()
				set_layout(parent)
			end

			def text()
				native.get_label()
			end

			def text=(val)
				native.set_label(val.to_s)
			end
		end
		Widgets['label'] = Label

		class ImageBox < WxWidget
			include WxWindowedWidget
			def create_widget(parent)
				@image = resource.image(properties['image']).new
				Wx::StaticBitmap.new(parent.native, -1, @image.native)
			end

			def init_widget(parent)
				super
				set_size()
				set_layout(parent)
			end

			def image()
				@image
			end

			def image=(val)
				native.set_bitmap(val.native)
				@image = val
			end
		end
		Widgets['imagebox'] = ImageBox

		class Text < WxWidget
			include WxWindowedWidget
			def initialize(resource, properties, children)
				super
				set_events('on_change')
			end

			def create_widget(parent)
				text = properties['text']
				text = '' unless text
				Wx::TextCtrl.new(parent.native, -1, text)
			end

			def init_widget(parent)
				super
				set_size()
				set_layout(parent)
				native.evt_text(native.get_id) { 
					events.fire('on_change', self, {'value'=> native.get_value()})
				}
			end

			def text()
				self.value
			end

			def text=(val)
				self.value = val
			end

			def value()
				native.get_value()
			end

			def value=(val)
				native.set_value(val.to_s)
			end

			def value_type
				'string'
			end
		end
		Widgets['text'] = Text

		class MultiText < Text
			include WxWindowedWidget
			def create_widget(parent)
				text = properties['text']
				text = '' unless text
				Wx::TextCtrl.new(parent.native, -1, text, Wx::Point.new(-1, -1), Wx::Size.new(-1, -1), Wx::TE_MULTILINE)
			end
		end
		Widgets['multitext'] = MultiText

		class Numeric < WxWidget
			include WxWindowedWidget
			def initialize(resource, properties, children)
				super
				set_events('on_change')
			end

			def create_widget(parent)
				text = properties['text']
				text = properties['value'] unless text
				text = '0' unless text
				@min = 0
				@min = properties['min'].to_i if properties['min']
				@max = 100
				@max = properties['max'].to_i if properties['max']
				Wx::SpinCtrl.new(parent.native, -1, text, Wx::Point.new(-1, -1), Wx::Size.new(-1, -1), Wx::SP_ARROW_KEYS, @min, @max)
			end
			attr_reader :min,:max

			def min=(val)
				@min = val.to_i
				native.set_range(@min, @max)
			end

			def max=(val)
				@max = val.to_i
				native.set_range(@min, @max)
			end

			def init_widget(parent)
				super
				set_size()
				set_layout(parent)
				on_change = proc { 
					val = native.get_value()
					if @min <= val && val <= @max then
						events.fire('on_change', self, {'value'=> native.get_value()})
					end
				}
				native.evt_spinctrl(native.get_id, &on_change)
				#native.evt_text(native.get_id, &on_change)
			end

			def value()
				val = native.get_value()
				val = @min if val<@min
				val = @max if val>@max
				return val
			end

			def value=(val)
				val = val.to_i
				if @min <= val && val <= @max then
					native.set_value(val)
				end
			end

			def value_type
				'integer'
			end
		end
		Widgets['numeric'] = Numeric

		class CheckBox < WxWidget
			include WxWindowedWidget
			def initialize(resource, properties, children)
				super
				set_events('on_change')
			end

			def create_widget(parent)
				label = properties['text']
				label = '' unless label
				Wx::CheckBox.new(parent.native, -1, label)
			end

			def init_widget(parent)
				super
				native.evt_checkbox(native.get_id()) {|event| 
					events.fire('on_change', self, {'value' => self.checked})
				}
				native.set_value(true) if Boolean.parse(properties['checked'])
				set_size()
				set_layout(parent)
			end

			def checked()
				self.value
			end

			def checked=(val)
				self.value = val
			end

			def value()
				native.get_value()
			end

			def value=(val)
				native.set_value(val ? true : false)
			end

			def value_type
				'boolean'
			end
		end
		Widgets['checkbox'] = CheckBox

		class RadioBox < WxWidget
			include WxWindowedWidget
			def initialize(resource, properties, children)
				super
				set_events('on_change')
			end
			
			def create_widget(parent)
				label = properties['text']
				label = '' unless label
				Wx::RadioButton.new(parent.native, -1, label)
			end

			def init_widget(parent)
				super
				native.evt_radiobutton(native.get_id()) {|event| 
					events.fire('on_change', self, {'value' => self.checked})
				}
				native.set_value(true) if Boolean.parse(properties['checked'])
				set_size()
				set_layout(parent)
			end

			def checked()
				self.value
			end

			def checked=(val)
				self.value = val
			end

			def value()
				native.get_value()
			end

			def value=(val)
				native.set_value(val ? true : false)
			end

			def value_type
				'boolean'
			end
		end
		Widgets['radiobox'] = RadioBox

		class List < WxWidget
			include WxWindowedWidget
			def initialize(resource, properties, children)
				super
				set_events('on_change')
			end
			public :children

			def create_widget(parent, style=nil)
				style = Wx::LB_SINGLE | Wx::LB_NEEDED_SB unless style
				Wx::ListBox.new(parent.native, -1, DefaultPosition, DefaultSize, nil, style)
			end

			def init_widget(parent, style=nil)
				super
				set_size()
				set_layout(parent)
				children.each do |child|
					child.attach_to(self)
				end
				self.selected = children[properties['selected'].to_i] if properties['selected']
				init_events()
			end

			def init_events()
				native.evt_listbox(native.get_id) { 
					events.fire('on_change', self, {'value'=>self.value, 'item'=>self.selected}) 
				}
			end
			
			def value()
				item = self.selected
				if item then
					item.value
				else
					nil
				end
			end

			def value=(val)
				item = val
				children.each do |child|
					item = child if child.value == val
				end
				self.selected = item
			end

			def value_type
				'string'
			end

			def selected()
				index = native.get_selection()
				return nil if index<0
				return children[index]
			end

			def selected=(item)
				index = children.index(item)
				if index then
					if native.get_selection()!=index then
						native.set_selection(index)
					end
				else
					native.set_selection(-1)
				end
			end

			def add(item)
				children.push(item)
				item.attach_to(self)
			end

			def remove(item)
				index = children.index(item)
				if index then
					children.delete_at(index)
					native.delete(index)
				end
			end

			def clear()
				children.size.times do 
					remove(children[0])
				end
			end

			def insert(positm, item)
				index = children.index(positm)
				if index then
					children.insert(index, item)
					item.attach_to(self, index)
				else
					self.add(item)
				end
			end

			def update()
				children.each_with_index do |child, i|
					native.set_string(i, child.text)
				end
			end
		end
		Widgets['list'] = List

		class MultiList < List
			def init_widget(parent)
				style = Wx::LB_NEEDED_SB
				if Boolean.parse(properties['simple']) then
					style |= Wx::LB_MULTIPLE
				else
					style |= Wx::LB_EXTENDED
				end
				super(parent, style)
			end

			def value()
				self.selected.collect {|item| item.value}
			end

			def value=(val)
				items = val
				if val.respond_to?(:collect) then
					items = val.collect { |v|
						item = v
						children.each do |child|
							item = child if child.value == val
						end
						item
					}
				else
					children.each do |child|
						items = child if child.value == val
					end
				end
				self.selected = items
			end

			def value_type
				'string'
			end

			def selected()
				return native.get_selections().collect {|i| children[i] }
			end

			def selected=(val)
				if defined? val.each then
					val.each do |item|
						super(item)
					end
				else
					super(val)
				end
			end
		end
		Widgets['multilist'] = MultiList

		class ListItem < Widget
			def initialize(resource, properties, children)
				super
				@text = properties['text']
				@text = '' unless @text
				@value = properties['value']
				@value = @text unless @value
			end
			attr :value, true
			attr :text, true

			def attach_to(parent, index=nil)
				if index then
					parent.native.insert_items([@text], index)
				else
					parent.native.append(@text)
				end
			end
		end
		Widgets['listitem'] = ListItem

		class CheckList < List
			def create_widget(parent, style=nil)
				style = Wx::LB_SINGLE | Wx::LB_NEEDED_SB unless style
				Wx::CheckListBox.new(parent.native, -1, DefaultPosition, DefaultSize, [], style)
			end

			def initialize(resource, properties, children)
				super
			end

			def init_widget(parent)
				super
				children.each_with_index do |child, i|
					native.check(i, Boolean.parse(child.properties['checked']))
				end
				native.evt_checklistbox(native.get_id()) {|event|
					events.fire('on_checked', self, {'value'=>self.checked})
				}
			end

			def check(item, val=true)
				index = children.index(item)
				if index then
					native.check(index, val)
				end
			end

			def add(item)
				super
				self.check(item, Boolean.parse(item.properties['checked']))
			end

			def insert(item, positm)
				super
				self.check(item, Boolean.parse(item.properties['checked']))
			end

			def uncheck(item)
				index = children.index(item)
				if index then
					native.check(index, false)
				end
			end

			def is_checked?(item)
				index = children.index(item)
				if index then
					return native.is_checked(index)
				else
					return nil
				end
			end

			def checked()
				children.values_at(*((0...native.get_count()).select {|i| native.is_checked(i) }))
			end

			def checked=(items)
				items.each do |item|
					self.check(item)
				end
			end
		end
		Widgets['checklist'] = CheckList
		
		class DropDownList < List
			def create_widget(parent, style=nil)
				Wx::ComboBox.new(parent.native, -1, '', DefaultPosition, DefaultSize, nil, style)
			end

			def init_widget(parent, style=nil)
				style = Wx::CB_DROPDOWN | Wx::CB_READONLY unless style
				super(parent, style)
			end

			def init_events()
				native.evt_combobox(native.get_id) { 
					events.fire('on_change', self, {'value'=>self.value, 'item'=>self.selected }) 
				}
			end
			
		end
		Widgets['dropdownlist'] = DropDownList

		class WxMenuItem
			class Items
				def initialize()
					@native = Wx::Menu.new()
					@frame = nil
				end
				attr :native
				attr :frame, true

				def add(item)
					item.frame = @frame
					item.parent = self
					if item.items then
						newitem = Wx::MenuItem.new(
							nil, item.id, item.text, 
							item.hint, Wx::ITEM_NORMAL, item.items.native)
						@native.append_item(newitem)
					else
						id = item.id
						id = Wx::ID_SEPARATOR if item.text=='-'
						@native.append(id, item.text, item.hint)
					end
				end
			end

			@@new_id = 0
			def initialize(text='-', hint=nil, has_children=false)
				@items = nil
				@items = Items.new() if has_children
				@text = text
				@hint = hint
				@id = @@new_id
				@@new_id += 1
				@frame = nil
				@parent = nil
			end
			attr_reader :items, :text, :hint, :id
			attr :parent, true

			def frame()
				@frame
			end

			def frame=(val)
				@frame = val
				@items.frame = val if @items
			end

			def enable(val)
				@parent.native.enable(@id, val)
			end
			def is_enabled()
				@parent.native.is_enabled(@id)
			end
		end
		
		class WxMainMenu
			class Items
				def initialize(menubar)
					@native = menubar
					@frame = nil
				end
				attr_reader :native
				attr :frame, true

				def add(item)
					item.frame = @frame
					item.parent = self
					if item.items then
						@native.append(item.items.native, item.text)
					else
						@native.append(Wx::Menu.new(), item.text)
					end
				end
			end

			def initialize()
				@native = Wx::MenuBar.new()
				@items = Items.new(@native)
			end
			attr_reader :native, :items

			def frame()
				@items.frame
			end

			def frame=(val)
				@items.frame = val
			end
		end

		class MainMenu < WxWidget
			def create_widget(parent)
				menu = WxMainMenu.new()
				parent.native.set_menu_bar(menu.native)
				menu.frame = parent.native
				return menu
			end
			
			def init_widget(parent)
				super
				children.each do |child|
					child.attach_to(self)
				end
			end

		end
		Widgets['mainmenu'] = MainMenu

		class MenuItem < WxWidget
			def initialize(resource, properties, children)
				super
				set_events('on_click')
			end

			def create_widget(parent)
				item = WxMenuItem.new(
					properties['text'].to_s, properties['hint'].to_s, children.size>0)
				parent.native.items.add(item)
				return item
			end

			def init_widget(parent)
				super
				parent.native.frame.evt_menu(native.id) { events.fire('on_click', self, {}) }
				self.enabled = Boolean.parse(properties['enabled']) if properties['enabled']
				children.each do |child|
					child.attach_to(self)
				end
			end

			def enabled=(val)
				native.enable(val)
			end

			def enabled()
				native.is_enabled()
			end
		end
		Widgets['menuitem'] = MenuItem

		class SplitterPanel < WxWidget
			include WxWindowedWidget
			def create_widget(parent)
				Wx::SplitterWindow.new(parent.native, -1, DefaultPosition, DefaultSize, Wx::SP_NOBORDER)
			end

			def init_widget(parent)
				super(parent)
				set_size(true)
				set_layout(parent)
				panel_children1 = []
				panel_children2 = []
				panel_children1.push(children[0]) if children.size>0 
				panel_children2.push(children[1]) if children.size>1
				warn 'Too many children for splitterpanel' if children.size>2
				subpanel1 = Panel.new(resource, {}, panel_children1)
				subpanel2 = Panel.new(resource, {}, panel_children2)
				subpanel1.attach_to(self)
				subpanel2.attach_to(self)
				pos = 0
				pos = Length.to_px(properties['split']) if properties['split']
				if Boolean.parse(properties['vertical']) then
					native.split_horizontally(subpanel1.native, subpanel2.native, pos)
				else
					native.split_vertically(subpanel1.native, subpanel2.native, pos)
				end
			end
		end
		Widgets['splitterpanel'] = SplitterPanel

		class ToolBar < WxWidget
			include WxWindowedWidget
			class WxToolBar < Wx::ToolBar
				def initialize(nparent, id, pos, size, style)
					super(nparent, id, pos, size, style)
					@tools = []
					@max_size = nil
				end

				def add_tool(tool)
					@tools.push(tool)
					if tool.text=='-' then
						add_separator()
						return -1
					else
						newid = @tools.size-1
						super(newid, tool.text, tool.image, tool.image, Wx::ITEM_NORMAL, tool.help, tool.help, tool)
						evt_tool(newid) {|*args|
							tool.event.call(*args) if tool.event
						}
						unless @max_size then
							sz = get_tool_bitmap_size()
							sz.set_width(tool.image.get_width())
							sz.set_height(tool.image.get_height())
							set_tool_bitmap_size(sz)
							@max_size = sz
						else
							if @max_size.get_width < tool.image.get_width() then
								@max_size.set_width(tool.image.get_width)
							end
							if @max_size.get_height < tool.image.get_height() then
								@max_size.set_height(tool.image.get_height)
							end
							set_tool_bitmap_size(@max_size)
						end
						realize
						return newid
					end
				end
			end

			def create_widget(parent)
				style = Wx::TB_HORIZONTAL | Wx::NO_BORDER | Wx::TB_FLAT | Wx::TB_DOCKABLE
				WxToolBar.new(parent.native, -1, DefaultPosition, DefaultSize, style)
			end

			def init_widget(parent)
				super
				set_size()
				set_layout(parent)
				children.each do |child|
					child.attach_to(self)
				end
			end
		end
		Widgets['toolbar'] = ToolBar

		class ToolButton < WxWidget
			class WxToolButton
				def initialize(parent, text, image, help='')
					super()
					@text = text
					@image = image
					@help = help
					@event = nil
					@parent = parent
					@id = @parent.add_tool(self)
				end
				attr_reader :text, :image, :help, :event

				def evt_tool(id, &block)
					@event = block
				end

				def get_id()
					return @id
				end

				def enable(val)
					@parent.enable_tool(@id, val)
				end

				def is_enabled()
					@parent.get_tool_enabled(@id)
				end
			end

			def initialize(resource, properties, children)
				super
				set_events('on_click')
			end

			def create_widget(parent)
				hint = properties['hint']
				hint = '' unless hint
				text = properties['text']
				text = '' unless text
				image = nil
				if properties['image'] then
					image = resource.image(properties['image']).new.native
				end
				WxToolButton.new(parent.native, text, image, hint)
			end

			def init_widget(parent)
				super
				native.evt_tool(native.get_id) { events.fire('on_click', self) }
				self.enabled = Boolean.parse(properties['enabled']) if properties['enabled']
			end

			def enabled=(val)
				native.enable(val)
			end

			def enabled()
				native.is_enabled()
			end
		end
		Widgets['toolbutton'] = ToolButton

		class FileOpenDialog < WxWidget
			def initialize(resource, properties, children)
				super
				set_events('on_select')
			end

			def create_widget(parent, style = 0, filter='*.*')
				pnative = parent ? parent.native : nil
				style |= Wx::OPEN
				Wx::FileDialog.new(pnative, 'Open', '', '', filter, style)
			end

			def init_widget(parent=nil)
				style = Wx::OPEN | Wx::HIDE_READONLY
				@multiple = Boolean.parse(properties['multiselect'])
				style |= Wx::MULTIPLE if @multiple
				filter = '*.*'
				filter = properties['filter'] if properties['filter']
				super(parent, style, filter)
				native.set_message(properties['text']) if properties['text']
				children.each do |child|
					child.attach_to(self)
				end
			end

			def path()
				native.get_path()
			end

			def path=(val)
				native.set_path(val.to_s)
			end

			def show()
				res = nil
				ret = native.show_modal()
				if ret==Wx::ID_OK then
					if @multiple then
						res = native.get_paths()
					else
						res = native.get_path()
					end
					events.fire('on_select', self, {'value'=>res})
				end
				return res
			end

			def close(ret = 0)
				native.end_modal(ret)
			end
		end
		Widgets['fileopendialog'] = FileOpenDialog

		class FileSaveDialog < WxWidget
			def initialize(resource, properties, children)
				super
				set_events('on_select')
			end

			def create_widget(parent, style = 0, filter='*.*')
				pnative = parent ? parent.native : nil
				style |= Wx::SAVE
				Wx::FileDialog.new(pnative, 'Save', '', '', filter, style)
			end

			def init_widget(parent=nil)
				style = Wx::SAVE | Wx::OVERWRITE_PROMPT
				filter = '*.*'
				filter = properties['filter'] if properties['filter']
				super(parent, style, filter)
				native.set_message(properties['text']) if properties['text']
				children.each do |child|
					child.attach_to(self)
				end
			end

			def path()
				native.get_path()
			end

			def path=(val)
				native.set_path(val.to_s)
			end

			def show()
				ret = native.show_modal()
				if ret==Wx::ID_OK then
					events.fire('on_select', self, {'value'=>native.get_path()})
				end
				return ret
			end

			def close(ret = 0)
				native.end_modal(ret)
			end
		end
		Widgets['filesavedialog'] = FileSaveDialog

		class DirSelectDialog < WxWidget
			def initialize(resource, properties, children)
				super
				set_events('on_select')
			end

			def create_widget(parent, style = 0x80 | Wx::RESIZE_BORDER | Wx::DEFAULT_DIALOG_STYLE)
				pnative = parent ? parent.native : nil
				msg = 'Choose a directory'
				msg = properties['text'] if properties['text']
				Wx::DirDialog.new(pnative, msg, '', style)
			end

			def init_widget(parent=nil)
				super(parent)
			end

			def path()
				native.get_path()
			end

			def path=(val)
				native.set_path(val.to_s)
			end

			def show()
				ret = native.show_modal()
				if ret==Wx::ID_OK then
					events.fire('on_select', self, {'value'=>native.get_path()})
				end
				return ret
			end

			def close(ret = 0)
				native.end_modal(ret)
			end
		end
		Widgets['dirselectdialog'] = DirSelectDialog

		class MessageBox
			def create_widget(parent, style = Wx::CENTRE | Wx::OK)
				pnative = parent ? parent.native : nil
				msg = 'Message'
				msg = properties['text'] if properties['text']
				title = 'Message'
				title = properties['title'] if properties['title']
				Wx::MessageDialog.new(pnative, msg, title, style)
			end

			Buttons = {'ok'=>Wx::OK, 'cancel'=>Wx::CANCEL, 'yes'=>Wx::YES_NO, 'no'=>Wx::YES_NO}
			Results = {Wx::ID_OK=>'ok', Wx::ID_CANCEL=>'cancel', Wx::ID_YES=>'yes', Wx::ID_NO=>'no'}
			def init_widget(parent=nil)
				style = Wx::OK | Wx::CENTRE
				if properties['buttons'] then
					style = properties['buttons'].split(' ').inject(Wx::CENTRE) {|res, item| res |= Buttons[item]}
				end
				super(parent, style)
			end

			def show()
				ret = native.show_modal()
				return Result[ret]
			end
		end
		Widgets['messagebox'] = MessageBox
	end
end

