$: << '../../'

require 'ridge'
require 'ridge/wxwindows'
require 'ridge/reader/rexml'

require 'pstore'

#Ridge.encoding = 'cp932'

AddressRecord = Struct.new(
	:first_name, :last_name,
	:age, :sex,
	:postal_code, :address,
	:company,
	:telephone1, :telephone2,
	:e_mail1, :e_mail2)

class AddressBookApp
	def initialize()
		@rc = Ridge::Resource.open('addressbook.xml')
		@main = @rc.widget('MainWindow').new()
		@records = []
		@current_record = AddressRecord.new()
		@bind = Ridge::GUI::Bind.new(@main, @current_record)
		@filename = nil

		@rc.events.add_handler('select_record', self.method(:select_record))
		@rc.events.add_handler('new_record', self.method(:new_record))
		@rc.events.add_handler('delete_record', self.method(:delete_record))
		@rc.events.add_handler('file_new') do
			@filename = nil
			self.records = []
			@current_record = AddressRecord.new()
			@bind.target = @current_record
			@bind.update()
		end
		@rc.events.add_handler('file_open') do
			@main.widget('OpenFileDlg').show()
		end
		@rc.events.add_handler('file_save') do
			if @filename then
				save()
			else
				@main.widget('SaveFileDlg').show()
			end
		end
		@rc.events.add_handler('file_save_as') do
			@main.widget('SaveFileDlg').show()
		end
		@rc.events.add_handler('file_quit') do 
			@main.close
		end
		@rc.events.add_handler('on_open') do |sender, args|
			open(args['value'])
		end
		@rc.events.add_handler('on_save') do |sender, args|
			save(args['value'])
		end
	end

	def records
		@records
	end

	def records=(val)
		@records = val
		list = @main.widget('RecordList')
		list.clear()
		@records.each do |rec|
			name = "#{rec.first_name}, #{rec.last_name}"
			name = '(no name)' if name==', '
			item = Ridge::GUI::ListItem.new(@rc, {'text'=>name, 'value'=>rec}, [])
			list.add(item)
		end
	end

	def update_list()
		list = @main.widget('RecordList')
		list.children.each do |child|
			rec = child.value
			name = "#{rec.first_name}, #{rec.last_name}"
			name = '(no name)' if name==', '
			child.text = name
		end
		list.update()
	end
	
	def new_record(sender, args)
		rec = AddressRecord.new()
		@records.push(rec)
		item = Ridge::GUI::ListItem.new(@rc, {'text'=>'(no name)', 'value'=>rec}, [])
		@main.widget('RecordList').add(item)
	end

	def delete_record(sender, args)
		item = @main.widget('RecordList').selected
		if item and item.value then
			@main.widget('RecordList').remove(item)
			@records.delete_if {|rec| rec.equal?(item.value) }
			@current_record = AddressRecord.new()
			@bind.target = @current_record
			@bind.update()
		end
	end

	def select_record(sender, args)
		@current_record = args['value']
		@bind.target = @current_record
		@bind.update()
		update_list()
	end

	def open(filename)
		if filename then
			begin
				db = PStore.new(filename)
				db.transaction() do
					self.records = db.fetch('Addressbook')
				end
				@filename = filename
			rescue
				Ridge::GUI.message("Can not open file #{filename}")
			end
		end
	end

	def save(filename=nil)
		filename = @filename unless filename
		if filename then
			begin
				db = PStore.new(filename)
				db.transaction() do
					db['Addressbook'] = self.records
				end
				@filename = filename
			rescue
				Ridge::GUI.message("Can not save file #{filename}")
			end
		end
	end

	def run()
		@main.run
	end
end

if $0==__FILE__ then
	AddressBookApp.new.run()
end

